'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.composeWithRelay = composeWithRelay;

var _graphqlCompose = require('graphql-compose');

var _nodeInterface = require('./nodeInterface');

var _nodeInterface2 = _interopRequireDefault(_nodeInterface);

var _mutationMiddleware = require('./mutationMiddleware');

var _mutationMiddleware2 = _interopRequireDefault(_mutationMiddleware);

var _globalId = require('./globalId');

var _graphql = require('graphql');

var _nodeFieldConfig = require('./nodeFieldConfig');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// all wrapped typeComposers with Relay, stored in this variable
// for futher type resolving via NodeInterface.resolveType method

/* eslint-disable no-use-before-define */

var typeComposerMap = {};

function composeWithRelay(typeComposer) {
  if (!(typeComposer instanceof _graphqlCompose.TypeComposer)) {
    throw new Error('You should provide TypeComposer instance to composeWithRelay method');
  }

  if (typeComposer.getTypeName() === 'RootQuery') {
    typeComposer.addField('node', (0, _nodeFieldConfig.getNodeFieldConfig)(typeComposerMap));
    return typeComposer;
  }

  if (typeComposer.getTypeName() === 'RootMutation') {
    // just skip
    return typeComposer;
  }

  if (!typeComposer.hasRecordIdFn()) {
    throw new Error('TypeComposer(' + typeComposer.getTypeName() + ') should have recordIdFn. ' + 'This function returns ID from provided object.');
  }

  var findById = typeComposer.getResolver('findById');
  if (!findById) {
    throw new Error('TypeComposer(' + typeComposer.getTypeName() + ') provided to composeWithRelay ' + 'should have findById resolver.');
  }
  typeComposerMap[typeComposer.getTypeName()] = findById;

  typeComposer.addFields({
    id: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID),
      description: 'The globally unique ID among all types',
      resolve: function resolve(source) {
        return (0, _globalId.toGlobalId)(typeComposer.getTypeName(), typeComposer.getRecordId(source));
      }
    }
  });

  typeComposer.addInterface(_nodeInterface2.default);

  typeComposer.getResolvers().forEach(function (resolver) {
    if (resolver.kind === 'mutation') {
      resolver.addMiddleware(new _mutationMiddleware2.default(typeComposer, resolver));
    }
  });

  return typeComposer;
}